"""Workflows to update the fluffy analysis run status"""

import logging

from fluffy.slurm_api import SlurmAPI

LOG = logging.getLogger(__name__)

def sed_replace_inplace(
    in_filename: str,project_dir:str ,flag: str, find_str: str, replace_str: str
) -> str:
    """create a command for running sed find and replace inplace"""
    cmd = (
        f"sed -i \'s/{find_str}/{replace_str}/g\' {in_filename}\n"
        f"echo {replace_str} > {project_dir}/{flag}"
    )
    return cmd

def pipe_complete(
    configs: dict,
    afterok: int,
    slurm_api: SlurmAPI,
    dry_run: bool = False,
) -> int:
    """Run Sed to update the analysis run flag"""
    out_dir = configs["out"]

    sed_replace_inplace_cmd = sed_replace_inplace(
        in_filename= str(out_dir / "analysis_status.json"),project_dir=str(out_dir),flag="COMPLETE", find_str=" \"running\"", replace_str=" \"complete\""
    )

    jobid = slurm_api.run_job(
        name="fluffy-complete",
        command=sed_replace_inplace_cmd,
        afterok=[afterok],
        dry_run=dry_run,
    )

    return jobid

def pipe_fail(
    configs: dict,
    slurm_api: SlurmAPI,
    afternotok: int,
    dry_run: bool = False,
) -> int:
    """Run Sed to update the analysis run flag"""
    out_dir = configs["out"]

    sed_replace_inplace_cmd = sed_replace_inplace(
        in_filename= str(out_dir / "analysis_status.json"),project_dir=str(out_dir),flag="FAIL", find_str=" \"running\"", replace_str=" \"fail\""
    )

    jobid = slurm_api.run_job(
        name="fluffy-fail",
        command=sed_replace_inplace_cmd,
        afternotok=[afternotok],
        dry_run=dry_run,
    )

    return jobid



"""Code to handle samplesheets"""

import logging
from pathlib import Path
from typing import Iterator

LOG = logging.getLogger(__name__)


def get_separator(line: str) -> str:
    """Get the separator for file"""
    if " " in line:
        return " "
    if "," in line:
        return ","
    return None


def get_sample_col(line_content: list) -> int:
    """Get the column number that holds the sample name"""
    for column, info in enumerate(line_content):
        if info.lower() == "sampleid":
            return column
    return None

def get_sample_name_col(line_content: list) -> int:
    """Get the column number that holds the sample name"""
    for column, info in enumerate(line_content):
        if info.lower() == "samplename":
            return column
    return None


def read_samplesheet(samplesheet: Iterator[str], project_dir: Path) -> Iterator[dict]:
    """Parse a sample sheet and return sample information

    Yields:
        samples(dict): a dictionary with a list of commands and 'se'(bool)
    """

    samples = set()
    sample_col = 0

    first=True
    for line_nr, line in enumerate(samplesheet):

        if line.startswith("[Data]") and first:
            continue

        if first:
            separator = get_separator(line)
            LOG.debug("Use separator %s", separator)
            header = line.rstrip().split(separator)
            sample_col = get_sample_col(header)
            sample_name_col= get_sample_name_col(header)
            first=False
            continue

        content = line.rstrip().split(separator)
        sample_name = content[sample_col]

        if sample_name in samples:
            continue

        samples.add(sample_name)

        single_end = True
        LOG.debug("Check if files are single end or not")
        for file_name in project_dir.glob(f"*{sample_name}*/*.fastq.gz"):
            if "_R2" in str(file_name):
                single_end = False
                break

        fastq = [
            "<( zcat {}/*{}/*_R1*fastq.gz )".format(project_dir, sample_name),
            "<( zcat {}/*{}/*_R2*fastq.gz )".format(project_dir, sample_name),
        ]
        if single_end:
            LOG.info("Single end files!")
            fastq = ["<( zcat {}/*{}/*_R1*fastq.gz )".format(project_dir, sample_name)]

        if sample_name_col:
           sample_name = content[sample_name_col]
        print(sample_name)
        yield {"fastq": fastq, "single_end": single_end, "sample_id": sample_name}



"""Commands to run picard tools"""


def get_collect_gc_bias_cmd(
    singularity: str, out_prefix: str, reference: str, javasettings: str, tmp_dir: str
) -> str:
    """Get a string with command for running picard GC bias"""
    picard_gc_cmd = (
        f"{singularity} picard CollectGcBiasMetrics I={out_prefix}.bam "
        f"O={out_prefix}.gc_bias_metrics.txt CHART={out_prefix}.gc_bias_metrics.pdf "
        f"S={out_prefix}.gc.summary.tab R={reference} VALIDATION_STRINGENCY=LENIENT {javasettings} TMP_DIR={tmp_dir}"
    )

    return picard_gc_cmd


def get_collect_insert_size_cmd(
    singularity: str, out_prefix: str, javasettings: str, tmp_dir: str
) -> str:
    """Get a string with command to run picard collect insert size metrics"""
    picard_insert_cmd = (
        f"{singularity} picard CollectInsertSizeMetrics I={out_prefix}.bam "
        f"O={out_prefix}.insert_size_metrics.txt H={out_prefix}.insert_size_histogram.pdf "
        f"VALIDATION_STRINGENCY=LENIENT M=0.5 {javasettings} TMP_DIR={tmp_dir}"
    )

    return picard_insert_cmd


def get_estimate_complexity_cmd(
    singularity: str, out_prefix: str, javasettings: str, tmp_dir: str
) -> str:
    """Get a string with command to estimate library complexity with picard tools"""
    picard_complexity_cmd = (
        f"{singularity} picard EstimateLibraryComplexity I={out_prefix}.bam "
        f"O={out_prefix}.complex_metrics.txt VALIDATION_STRINGENCY=LENIENT {javasettings} TMP_DIR={tmp_dir}"
    )

    return picard_complexity_cmd



"""Code to align reads of a sample on SLURM"""

import logging

from fluffy.commands.pipelines import (align_and_convert_paired_end,
                                       align_and_convert_single_end)
from fluffy.slurm_api import SlurmAPI

LOG = logging.getLogger(__name__)


def align_individual(
    configs: dict, sample: dict, slurm_api: SlurmAPI, dry_run: bool = False,
):
    """Align a individual with bwa on slurm"""
    out_dir = configs["out"]
    sample_id = sample["sample_id"]
    LOG.info("Aligning reads for %s", sample_id)
    single_end = sample["single_end"]
    fastq = sample["fastq"]

    if single_end:
        run_bwa = align_and_convert_single_end(
            config=configs, fastq=fastq[0], out=out_dir,sample_id=sample_id
        )
    else:
        run_bwa = align_and_convert_paired_end(
            config=configs, fastq=fastq, out=out_dir, sample_id=sample_id
        )

    align_jobid = slurm_api.run_job(
        name=f"bwaAln-{sample_id}", command=run_bwa, dry_run=dry_run
    )

    return align_jobid
